/**
 * Module Dependencies
 *
 * @ignore
 */
const bodyParser = require('body-parser')
const express = require('express')
const logger = require('structure-logger')
const md5 = require('md5')
const path = require('path')
const Raven = require('raven')
const Router = require('structure-router')
const serveStatic = require('serve-static')
const WebsocketServer = require('structure-websocket-server')

const USE_SENTRY = (process.env.SENTRY_DSN) ? true : false

if(USE_SENTRY) {
  Raven.config(process.env.SENTRY_DSN, {
    captureUnhandledRejections: true,
    environment: process.env.SENTRY_ENV,
    release: process.env.BUILD_NUMBER,
    parseUser: function parseUser(req) {
      return {
        xRequestId: req.headers['x-request-id']
      }
    }
  }).install()
}

function removePoweredBy(req, res, next) {
  res.removeHeader('X-Powered-By')
  next()
}

function setRaven(req, res, next) {
  req.Raven = Raven

  next()
}

/**
 * Server Class
 *
 * @public
 * @class Server
 */
class Server {

  /**
   * Server constructor
   *
   * @public
   * @constructor
   * @param {Object} options - Options
   */
  constructor(options = {}) {

    var defaults = {
      cursors: options.cursors || [],
      db: true,
      drain: true,
      sockets: true
    }

    this.options = Object.assign({}, defaults, options)
    this.pkgJson = options.pkgJson || ''
    this.server = express()

    if(USE_SENTRY) {
      this.server.use(Raven.requestHandler())
      this.server.use(Raven.errorHandler())
      this.server.use(setRaven)
    }

    this.server.use(serveStatic(path.join(__dirname, '../public')))
    this.server.use(bodyParser.urlencoded({extended: true}))
    this.server.use(bodyParser.json({strict: false}))

    this.router = options.router || new Router({
      middleware: options.middleware,
      routes: options.routes
    })

    this.router.start(this.server)

    let routes = this.router.routesList

    // For debuggin
    this.routesList = routes

    this.server.get(`/api/${process.env.API_VERSION}`, function routesListHandler(req, res) {

      res.json({
        routes
      })

    })

  }

  closeCursors() {
    var cursors = require('./cursors')

    for(let i = 0, l = cursors.length; i < l; i++) {
      var cursor = cursors[i]
      cursor.close()
    }

    cursors = []

  }

  /**
   * Start the HTTP Server
   *
   * @public
   */
  start() {

    process.env.STRUCTURE_HASH = md5(this.pkgJson)

    var httpServer = require('http').createServer(this.server);

    if(this.options.socketHandlers) {
      this.wss = new WebsocketServer(httpServer, this.options.socketHandlers)
    }

    this.server = httpServer.listen(this.options.port || process.env.EXPRESS_PORT)

    if(this.options.db) {
      this.cursors = []
    }

    logger.debug('Structure API started at:', `http://localhost:${process.env.EXPRESS_PORT}`)

  }

  /**
   * Stop the HTTP Server
   *
   * @public
   */
  stop() {

    if(this.options.db) {
      var r = require('structure-driver')
      if(this.options.drain) r.getPoolMaster().drain()
      this.closeCursors()
    }

    if(this.options.sockets) {
      this.wss.stop()
    }

    this.server.close()

  }

  /**
   * Add Express middleware to the Express server object
   *
   * @public
   */
  use() {

    this.server.use.apply(this.server, arguments)

  }

}

module.exports = Server
