const path = require('path')
const pluginsInterface = require('structure-plugins')
const Router = require('structure-router')
const WebsocketServer = require('structure-websocket-server')
const http = require('http')
const Server = require('../../index')

/** @test {Server} */
describe('HTTP Server', function() {

  beforeEach(function() {

    this.websocketSpy = sinon.spy()
    this.httpServerSpy = sinon.spy()
    this.httpServerSpy.listen = sinon.spy()

    this.sandbox = sinon.sandbox.create()
    this.sandbox.stub(http, 'createServer').returns(this.httpServerSpy)
    this.sandbox.stub(WebsocketServer.prototype, 'createWebsocketServer').callsFake(
      this.websocketSpy
    )
  })

  afterEach(function() {

    this.websocketSpy.reset()
    this.httpServerSpy.reset()
    this.httpServerSpy.listen.reset()
    this.sandbox.restore()

  })

  it('should initialize', function() {

    var api = new Server()

    expect(api).to.be.an('object')
    expect(api.server).to.be.an('function') // Express server
    expect(api.router).to.be.an('object')
    expect(api.start).to.be.a('function')
    expect(api.stop).to.be.a('function')
    expect(api.use).to.be.a('function')

  })

  /** @test {Server#start} */
  it('should start', function() {

    var api = new Server()

    api.start()

    expect(this.httpServerSpy.listen.calledOnce).to.be.true
    expect(this.websocketSpy.calledOnce).to.be.false

  })

  /** @test {Server#start} */
  it('should start websocket server if socketHandlers supplied', function() {

    const socketHandlers = [{
      eventName: 'test',
      handler: function() {}
    }]

    var api = new Server({
      socketHandlers
    })

    api.start()

    expect(this.httpServerSpy.listen.calledOnce).to.be.true
    expect(this.websocketSpy.calledOnce).to.be.true
    expect(this.websocketSpy.args[0]).to.deep.equal([
      this.httpServerSpy,
      socketHandlers
    ])

  })

  /** @test {Server#stop} */
  it('should stop', function() {

    var api = new Server()
    sinon.stub(api.server, 'listen').returns(api.server)
    sinon.stub(api, 'stop')

    api.start()
    api.stop()

    expect(api.stop.calledOnce).to.be.true

  })

  it.skip('should prevent query parameter pollution', async function() {

    var pkg = {
      title: 'My Template'
    }

    var res = await new MockHTTPServer()
      .post(`/api/v${process.env.API_VERSION}/templates?desc=cool&desc=cooler`)
      .send(pkg)

    expect(res.body.req.queryPolluted).to.be.true
    expect(res.body.req.query.desc).to.equal('cooler')

  })

  it.skip('should generate routes list', function() {

    const loaded = pluginsInterface({
      Controller: class RootController {},
      Model: class RootModel {},
      list: [
        path.join(__dirname, '../helpers/plugin1')
      ]
    })

    const api = new Server({
      router: new Router({
        routes: loaded.routes
      })
    })

    expect(api.routesList.length > 0).to.be.true

  })

})
